const fs = require('fs');
const ini = require('ini');
const mastodon = require('mastodon');
const mqtt = require('mqtt');
const request = require('request');
const rgbcolor = require('rgb-color');
const xmpp = require('simple-xmpp');

try {
	fs.readFileSync('./config.ini', 'utf-8');

const config = ini.parse(fs.readFileSync('./config.ini', 'utf-8'));
const debugging = config.debug;

// Asterisk config
const asterisk_callfrom = config.asterisk.callfrom;
const asterisk_callerID = config.asterisk.callerID;
const asterisk_callto = config.asterisk.callerto;
const asterisk_context = config.asterisk.context;
const asterisk_host = config.asterisk.host;
const asterisk_password = config.asterisk.password;
const asterisk_port = config.asterisk.port;
const asterisk_user = config.asterisk.user;

// Bot config
const bot_hostname = config.bot.hostname;
const bot_jid = config.bot.jid;
const bot_password = config.bot.password;
const bot_port = config.bot.port;

// Gotify config
const gotify_api_url = config.gotify.api_url;
const gotify_key = config.gotify.key;
const gotify_message = config.gotify.message;
const gotify_priority = config.gotify.priority;
const gotify_title = config.gotify.title;

// Mastodon config
const mastodon_api_url = config.mastodon.api_url;
const mastodon_token = config.mastodon.token;
const mastodon_toot = config.mastodon.toot;

// MQTT config
const mqtt_message = config.mqtt.message;
const mqtt_password = config.mqtt.password;
const mqtt_server = config.mqtt.server;
const mqtt_topic = config.mqtt.topic;
const mqtt_user = config.mqtt.user;

// Tuya (Smart Lights) config
const tuya_name = config.tuya.name;
const tuya_bright_url = config.tuya.bright_url;
const tuya_color_url = config.tuya.color_url;
const tuya_status_url = config.tuya.status_url;
const tuya_toggle_url = config.tuya.toggle_url;

// Unlocks API config
const unlocks_api_url = config.unlocks.api_url;
const unlocks_pass = config.unlocks.pass;
const unlocks_user = config.unlocks.user;

// User config (authorized user)
const user_jid = config.user.jid //.split('/')[0];
const user_name = config.user.name;
const user_server = config.user.server;

// Weather config
const weather_key = config.weather.key;
const weather_city_code = config.weather.city_code;
const weather_city_name = config.weather.city_name;
const weather_units = config.weather.units;

// Private channel creation (uses friends secion of config.ini to invite them to a private chat)
const xmpp_api_url = config.xmpp.api_url;
const xmpp_auth = config.xmpp.key;
const xmpp_muc_name = config.xmpp.muc_name;
const xmpp_muc_password = config.xmpp.muc_password;
const xmpp_muc_server= config.xmpp.muc_server;

let pc = 'This is a private command. Please see https://wiki.vanderwarker.family/doku.php?id=code:vxmppb:start for your own instance!';
let weather_url = `https://api.openweathermap.org/data/2.5/weather?id=${weather_city_code}&appid=${weather_key}&units=${weather_units}`

var ami = new require('asterisk-manager')(asterisk_port, asterisk_host, asterisk_user, asterisk_password, false);
var client = mqtt.connect('tcp://' + mqtt_user + ':' + mqtt_password + '@' + mqtt_server);
var M = new mastodon({
	access_token: `${mastodon_token}`,
	timeout_ms: 60*1000,
	api_url: `${mastodon_api_url}`,
})

// thx: https://stackoverflow.com/questions/48750528/get-direction-from-degrees
function getDirection(angle) {
    var directions = ['North', 'North-East', 'East', 'South-East', 'South', 'South-West', 'West', 'North-West'];
    var index = Math.round(((angle %= 360) < 0 ? angle + 360 : angle) / 45) % 8;
    return directions[index];
}

// Once we're online, we'll send a message alerting admins
xmpp.on('online', function(data, to) {
	console.log('Connected with JID: ' + bot_jid);
	xmpp.send(user_jid, "[ONLINE]", false);
	client.publish(mqtt_topic, mqtt_message);
	M.post('statuses', { status: `${mastodon_toot}` });
});

xmpp.on('chat', function(from, message) {
  function debug(){
    if (debugging === '1'){
      console.log('-------------');
      console.log(message);
      console.log(from);
      console.log('-------------');
   };
  }
const friends = Object.keys(config.friends);
let gotifyURL  = `${gotify_api_url}/message?token=${gotify_key}&message=${from} ${gotify_message}&title=${gotify_title}&priority=${gotify_priority}`;
let gotifyEUrl = encodeURI(gotifyURL);
	// HELP
	if (message === '!help') {
		xmpp.send(from, 'https://wiki.vanderwarker.family/doku.php?id=code:vxmppb:commands');
		debug();
	}

	// UNLOCKS
	else if (message === '!unlocks') {
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
		var d = new Date();
		var weekday = new Array(7);
		weekday[0] =  "Sunday";
		weekday[1] = "Monday";
		weekday[2] = "Tuesday";
		weekday[3] = "Wednesday";
		weekday[4] = "Thursday";
		weekday[5] = "Friday";
		weekday[6] = "Saturday";
		var n = weekday[d.getDay()];

		request(unlocks_api_url + "?columns=" + n, (error, res, body) => {
		if (error) {
			return  console.log(error)
		};

		if (!error && res.statusCode == 200) {
			var unlockBody = new RegExp(':(.*)}');
			var unlockBodyMatch = body.match(unlockBody);
			const unlockFinalMessage = user_name + " has " + unlockBodyMatch[1] + " unlocks for " + n;
			xmpp.send(from, unlockFinalMessage);
			debug();
		}
		if (!error && res.statusCode == 401) {
			console.log('Invalid credentials');
			xmpp.send(from, 'Invalid credentials');
		}
		if (!error && res.statusCode == 404) {
			console.log('Record not found');
			xmpp.send(from, 'Record not found');
		}
		if (!error && res.statusCode == 500) {
			console.log('Server error.');
			xmpp.send(from, 'Server error. Please ping admin');
		}
	});
	debug();
}}
        // Tuya light control
	else if (message === '!lights') {
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
		request(tuya_status_url,(error, res, body) => {
		if (error) {
			return  console.log(error)
		};

		if (!error && res.statusCode == 200) {
			if (res.body === "true"){
				const stat = tuya_name + ": On";
				xmpp.send(from, stat);
				console.log(stat);
			} else if (res.body === "false") {
				const stat = tuya_name + ": Off";
				xmpp.send(from, stat);
				console.log(stat);
			} else {
				xmpp.send(from, "Unknown reply from api call. Please reply ping to reach an admin.");
				console.log("Tuya API: Unknown reply!");
			}
		}
		if (!error && res.statusCode == 401) {
			console.log('Invalid credentials');
			xmpp.send(from, 'Invalid credentials');
		}
		if (!error && res.statusCode == 404) {
			console.log('Record not found');
			xmpp.send(from, 'Record not found');
		}
		if (!error && res.statusCode == 500) {
			console.log('Server error.');
			xmpp.send(from, 'Server error. Please ping admin');
		}
	});
	debug();
}}
		else if (message === tuya_name + ' on') {
			if (from != user_jid){
				xmpp.send(from, pc);
			} else {
		request(tuya_toggle_url + "?stat=true&devname=" + tuya_name,(error, res, body) => {
		if (error) {
			return  console.log(error)
		};

		if (!error && res.statusCode == 200) {
				const stat = tuya_name + " has been turned on";
				xmpp.send(from, stat);
				console.log(stat);
		}
		if (!error && res.statusCode == 401) {
			console.log('Invalid credentials');
			xmpp.send(from, 'Invalid credentials');
		}
		if (!error && res.statusCode == 404) {
			console.log('Record not found');
			xmpp.send(from, 'Record not found');
		}
		if (!error && res.statusCode == 500) {
			console.log('Server error.');
			xmpp.send(from, 'Server error. Please ping admin');
		}
	});
	debug();
}}
	else if (message === tuya_name + ' off') {
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
		request(tuya_toggle_url + "?stat=false&devname=" + tuya_name,(error, res, body) => {
		if (error) {
			return  console.log(error)
		};

		if (!error && res.statusCode == 200) {
				const stat = tuya_name + " has been turned off";
				xmpp.send(from, stat);
				console.log(stat);
		}
		if (!error && res.statusCode == 401) {
			console.log('Invalid credentials');
			xmpp.send(from, 'Invalid credentials');
		}
		if (!error && res.statusCode == 404) {
			console.log('Record not found');
			xmpp.send(from, 'Record not found');
		}
		if (!error && res.statusCode == 500) {
			console.log('Server error.');
			xmpp.send(from, 'Server error. Please ping admin');
		}
	});
	debug();
}}
	else if (message.includes(tuya_name + ' brightness')) {
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
	  var per=message.substring(message.indexOf("brightness") + 10);
	  var perc=(per*10)
		request(tuya_bright_url + "?brite=" + perc + "&devname=" + tuya_name,(error, res, body) => {
		if (error) {
			return  console.log(error)
		};

		if (!error && res.statusCode == 200) {
				xmpp.send(from, tuya_name + "set to " + per + "%");
				console.log(tuya_name + "set to " + per + "%");
		}
		if (!error && res.statusCode == 401) {
			console.log('Invalid credentials');
			xmpp.send(from, 'Invalid credentials');
		}
		if (!error && res.statusCode == 404) {
			console.log('Record not found');
			xmpp.send(from, 'Record not found');
		}
		if (!error && res.statusCode == 500) {
			console.log('Server error.');
			xmpp.send(from, 'Server error. Please ping admin');
		}
	});
	if (config.debug === '1'){
	console.log('lights turned to ' + per + "%");}
}}
	else if (message.includes(tuya_name + ' color')) {
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
	  request(tuya_toggle_url + "?stat=true&devname=" + tuya_name,(error, res, body) => {})
	  var color=message.substring(message.indexOf("color") + 6);
	  var colorRgb=rgbcolor(color);
	  console.log(color)
	  if (colorRgb.isValid()) {
            var obj = colorRgb.channels();
            console.log(obj.r + ', ' + obj.g + ', ' + obj.b);
          }
		request(tuya_color_url + "?r=" + colorRgb.r + "&g=" + colorRgb.g + "&b=" + colorRgb.b + "&devname=" + tuya_name,(error, res, body) => {
		if (error) {
			return  console.log(error)
		};

		if (!error && res.statusCode == 200) {
				xmpp.send(from, tuya_name + "set to " + color);
				console.log(tuya_name + "set to " + color);
		}
		if (!error && res.statusCode == 401) {
			console.log('Invalid credentials');
			xmpp.send(from, 'Invalid credentials');
		}
		if (!error && res.statusCode == 404) {
			console.log('Record not found');
			xmpp.send(from, 'Record not found');
		}
		if (!error && res.statusCode == 500) {
			console.log('Server error.');
			xmpp.send(from, 'Server error. Please ping admin');
		}
	});
	if (config.debug === '1'){
	  console.log(tuya_name + "set to " + color);}
	}}
	// Ping a user to let them know you would like to communicate
	else if (message === '!ping') {
		xmpp.send(from, "Pinging " + user_name + "! Please stand by.");
		xmpp.send(user_jid, from + " has pinged you!");
		client.publish(mqtt_topic, from + ' has pinged you!');
		request.post(gotifyEUrl, function (err, response, body) {
			if(err){
				console.log('error:', err);
			}
		});
		ami.action({
			'action':'originate',
			'channel':'SIP/' + asterisk_callfrom,
			'context': asterisk_context,
			'callerID' : asterisk_callerID,
			'data': 'googletts.agi, \"hello. ' + from + ' has pinged you!\"',
			'application': 'agi'
		});
		debug();
	}

	// Check the weather using weatherunderground API
	else if (message === '!weather') {
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
		xmpp.send(from, "Checking weather. Please hold...\r\n \r\n");
		request(weather_url, function (err, response, body) {
			if(err){
				console.log('error:', err);
			} else {
				let w = JSON.parse(body);
				let wm = weather_city_name + " Weather: \r\n Tempature: " + w.main.temp + " degrees. \r\n Conditions: " + w.weather[0].main + "\r\n Pressure: " +
				w.main.pressure + " \r\n Wind Direction: " + getDirection(w.wind.deg) + "\r\n Wind Speed: " + w.wind.speed + " m.p.h. \r\n";
				let wmc = weather_city_name + " Weather: Tempature: " + w.main.temp + " degrees. Conditions: " + w.weather[0].main + ". Pressure: " +
				w.main.pressure + ". Wind Direction: " + getDirection(w.wind.deg) + ". Wind Speed: " + w.wind.speed + " miles per hour \r\n";
				client.publish('weather/temp', 'w.main.temp');
				client.publish('weather/conditions', 'w.weather[0].main');
				client.publish('weather/pressure', 'w.main.pressure');
				client.publish('weather/wind/direction', 'getDirection(w.wind.deg)');
				client.publish('weather/wind/speed', 'w.wind.speed');
				xmpp.send(from, wm);
				/*
				ami.action({
					'action':'originate',
					'channel':'SIP/' + asterisk_callfrom,
					'context': asterisk_context,
					'callerID' : asterisk_callerID,
					'data': 'googletts.agi,' + wmc,
					'application': 'agi'
				});
				*/
			}
		});
		debug();
	}}

	// Sends a nice goodbye message
	else if (message === '!bye'){
		xmpp.send(from, "Bye, have a beautiful time!");
		debug();
	}


	// Share-A-Command (#13?)
	else if (message === '!meet'){
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
			const createRoom = {
			  method: 'POST',
			  url: xmpp_api_url + '/create_room_with_opts',
			  headers: {
			    'Content-Type': 'application/json',
			    Authorization: xmpp_auth
			  },
			  body: {
			    name: xmpp_muc_name,
			    service: xmpp_muc_server,
			    host: bot_hostname,
			    options: {
			      allow_change_subj: 'false',
			      allow_private_messages: 'false',
			      allow_private_messages_from_visitors: 'false',
			      allow_visitor_nickchange: 'false',
			      allow_visitor_status: 'false',
			      lang: 'en',
			      mam: 'true',
			      members_only: 'true',
			      persistent: 'true',
			      public: 'false',
			      public_list: 'false',
			      title: 'Secure channel generated by https://vxmppb.tk',
			      allow_subscription: 'true'
			    }
			  },
			  json: true
			};
		request(createRoom, function (error, response, body) {
		  if (error) throw new Error(error);
		  console.log(body);
		});
		setTimeout(() => {
		for (const key in config.friends) {
		  xmpp.send(key, "Meeting started: " + xmpp_muc_name + "@" + xmpp_muc_server);
			const addMember = {
			  method: 'POST',
			  url: xmpp_api_url + 'set_room_affiliation',
			  headers: {
			    'Content-Type': 'application/json',
			    Authorization: xmpp_auth
			  },
			  body: {
			    name: xmpp_muc_name,
			    service: xmpp_muc_server,
			    jid: key,
			    affiliation: 'member'
			  },
			  json: true
			};
		const inviteMember = {
		  method: 'POST',
		  url: xmpp_api_url + '/send_direct_invitation',
	          headers: {
	            'Content-Type': 'application/json',
	            Authorization: xmpp_auth
	          },
		  body: {
		    name: xmpp_muc_name,
		    service: xmpp_muc_server,
		    password: '',
		    reason: 'vXMPPb Meeting Invite',
		    users: key
		  },
		  json: true
		};
		request(addMember, function (error, response, body) {
		  if (error) throw new Error(error);
		  console.log(body);
		});
		request(inviteMember, function (error, response, body) {
		  if (error) throw new Error(error);
		  console.log(body);
		});
		}}, 2000);
///////////////////////////////////////
		debug();
	}}

	// End meeting
	else if (message === '!endmeet'){
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
		const endMeeting = {
		  method: 'POST',
		  url: xmpp_api_url + '/destroy_room',
		  headers: {
		    'Content-Type': 'application/json',
		    Authorization: xmpp_auth
		  },
		  body: {name: xmpp_muc_name, service: xmpp_muc_server},
		  json: true
		};
		request(endMeeting, function (error, response, body) {
		  if (error) throw new Error(error);
		  console.log(body);
		});
			xmpp.send(from, "Meeting ended");
		for (const key in config.friends) {
			xmpp.send(key, "Meeting ended");
		}
	}debug();}



	// Shutsdown bot if admin sends shutdown command (linux)
	else if (message === "sudo shutdown now -h"){
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
			xmpp.send(from, "Alrighty, shutting down. Goodbye!");
			debug();
			process.exit(1);
	}}

	// Get users vCard (WIP)
	else if (message === '!vcard'){
		if (from != user_jid){
			xmpp.send(from, pc);
		} else {
		xmpp.getVCard(from, function (vcard) {
			xmpp.send(from, vcard);
			console.log(vcard)
		})
	}}

	// If the user sends something we don't have logic for, send them info about the project
	else {
		xmpp.send(from, "This is an auto replying bot. Please see https://wiki.vanderwarker.family/doku.php?id=code:vxmppb:commands");
	  debug();
	}
});

//  Catch and log XMPP errors
xmpp.on('error', function(err) {
console.error(err);
});

// Set XMPP status
xmpp.setPresence('chat', 'Asterisk,Mastodon, and MQTT. Oh my!');

// Alas, we connect to XMPP!
xmpp.connect({
	jid: bot_jid,
	password: bot_password,
	host: bot_hostname,
	port: bot_port
});

// Checks the XMPP server for new 'friend' requests
console.log(xmpp.getRoster());


 } catch(e) {
 	console.log("Please create a config.ini");
 	console.log("See https://wiki.vanderwarker.family/doku.php?id=code:vxmppb:install:start");
 	console.log("and choose your prefered install method.")
 }
